package kaysaar.aotd_question_of_loyalty.data.scripts.commision;

import java.util.LinkedHashMap;
import kaysaar.aotd_question_of_loyalty.data.intel.AoTDCommIntelPlugin;
import org.json.JSONObject;
import org.json.JSONArray;
import java.util.Set;
import kaysaar.aotd_question_of_loyalty.data.misc.QoLMisc;
import com.fs.starfarer.api.Global;
import java.util.List;
import java.util.Iterator;
import java.util.ArrayList;
import kaysaar.aotd_question_of_loyalty.data.models.BaseFactionCommisionData;
import kaysaar.aotd_question_of_loyalty.data.models.RankData;
import java.util.HashMap;

public class AoTDCommissionDataManager
{
    private static AoTDCommissionDataManager instance;
    private HashMap<String, RankData> availableRanks;
    private HashMap<String, BaseFactionCommisionData> commissionData;
    transient ArrayList<Object> listeners;
    
    public static AoTDCommissionDataManager getInstance() {
        if (AoTDCommissionDataManager.instance == null) {
            (AoTDCommissionDataManager.instance = new AoTDCommissionDataManager()).initalizeRanks();
            AoTDCommissionDataManager.instance.initalizeData();
            AoTDCommissionDataManager.instance.listeners = new ArrayList<Object>();
        }
        return AoTDCommissionDataManager.instance;
    }
    
    public boolean hasListenerOfClass(final Class<?> c) {
        for (final Object listener : this.listeners) {
            if (listener.getClass().equals(c)) {
                return true;
            }
        }
        return false;
    }
    
    public <T> List<T> getListeners(final Class<T> c) {
        final List<T> toReturn = new ArrayList<T>();
        for (final Object listener : this.listeners) {
            if (c.isInstance(listener)) {
                toReturn.add(c.cast(listener));
            }
        }
        return toReturn;
    }
    
    public void addListener(final Object listener) {
        this.listeners.add(listener);
    }
    
    public void initalizeRanks() {
        this.availableRanks = new HashMap<String, RankData>();
        try {
            final JSONArray array = Global.getSettings().getMergedSpreadsheetData("rank_id", "data/campaign/aotd_rank_data.csv");
            for (int i = 0; i < array.length(); ++i) {
                final JSONObject entry = array.getJSONObject(i);
                final String id = entry.getString("rank_id");
                if (QoLMisc.isStringValid(id, (String)null)) {
                    if (!id.contains("#")) {
                        final String name = entry.getString("name");
                        final Integer salary = entry.getInt("salary");
                        final boolean canLeave = entry.getBoolean("canLeave");
                        final int tariffReduction = entry.getInt("tariffReduction");
                        final int allowedMarketsToOwn = entry.getInt("allowedMarketsToOwn");
                        final int pointsOfObligation = entry.getInt("pointsOfObligation");
                        final int rawPenalty = entry.getInt("penaltyForLeavingRep");
                        final float truePenalty = rawPenalty / 100.0f;
                        final Set<String> tags = QoLMisc.loadEntriesSet(entry.getString("tags"), ",");
                        final String description = entry.getString("description");
                        this.availableRanks.put(id, new RankData(id, name, canLeave, truePenalty, description, pointsOfObligation, (int)salary, (float)tariffReduction, (Set)tags, allowedMarketsToOwn));
                    }
                }
            }
        }
        catch (final Exception e) {
            throw new RuntimeException(e);
        }
    }
    
    public void initalizeData() {
        this.commissionData = new HashMap<String, BaseFactionCommisionData>();
        try {
            final JSONArray array = Global.getSettings().getMergedSpreadsheetData("faction_id", "data/campaign/aotd_commision_data.csv");
            for (int i = 0; i < array.length(); ++i) {
                final JSONObject entry = array.getJSONObject(i);
                final String id = entry.getString("faction_id");
                if (QoLMisc.isStringValid(id, (String)null)) {
                    if (!id.contains("#")) {
                        final ArrayList<String> ranks = QoLMisc.loadEntriesArray(entry.getString("ranks"), ",");
                        final ArrayList<String> rawThreshold = QoLMisc.loadEntriesArray(entry.getString("rank_threshold"), ",");
                        final LinkedHashMap<String, String> iconMap = null;
                        if (QoLMisc.isStringValid(entry.getString("iconMap"))) {}
                        final HashMap<String, Integer> ranksOrdered = new HashMap<String, Integer>();
                        for (final String rank : rawThreshold) {
                            final String[] splitted = rank.split(":");
                            ranksOrdered.put(splitted[0].trim(), Integer.parseInt(splitted[1]));
                        }
                        final int maxPoints = entry.getInt("maxPoints");
                        String plugin = "";
                        if (QoLMisc.isStringValid(entry.getString("commisionPlugin"))) {
                            plugin = entry.getString("commisionPlugin");
                        }
                        else {
                            plugin = AoTDCommIntelPlugin.class.getName();
                        }
                        final ArrayList<RankData> ranksData = new ArrayList<RankData>();
                        for (final String rank2 : ranks) {
                            ranksData.add(this.getRank(rank2.trim()));
                        }
                        final Set<String> tags = QoLMisc.loadEntriesSet(entry.getString("tags"), ",");
                        this.commissionData.put(id, new BaseFactionCommisionData(id, (LinkedHashMap)iconMap, maxPoints, (ArrayList)ranksData, plugin, (HashMap)ranksOrdered, (Set)tags));
                    }
                }
            }
        }
        catch (final Exception e) {
            throw new RuntimeException(e);
        }
    }
    
    public RankData getRank(final String rankID) {
        return this.availableRanks.get(rankID);
    }
    
    public void addRank(final RankData rankData) {
        this.availableRanks.put(rankData.id, rankData);
    }
    
    public BaseFactionCommisionData getCommisionData(final String id) {
        if (this.commissionData.get(id) == null) {
            return BaseFactionCommisionData.getDefaultCommisionData(id);
        }
        return this.commissionData.get(id);
    }
}
